/*
Still picture viewer for the Technotrend DVB-s PCI card.

Copyright (C) 2001  Jan Panteltje
	jan@panteltje.demon.nl

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <pwd.h>
#include <string.h>

#include "ppv.h"

//#include <pthread.h>
//#include <sys/poll.h>


int debug_flag;
char *home_dir;
struct timeval *start_timeval;
struct timeval *current_timeval;


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

/* argument check */
if(! s) return 0;

p = malloc(strlen(s) +  1);
if(p) strcpy(p, s);
return(p);
}/* end function strsave */


char *strcasestr(char *haystack, char *needle)
{
/* case independent strstr */
char *hptr;
char *nptr;
char *pptr;

/* start at the beginning of haystack */
pptr = haystack;

/* for each position in haystack */
while(1)
	{
	/* starting at this position in haystack */
	hptr = pptr;
	nptr = needle;

	/* test if strings match from here on */
	while(1)
		{
		/* test for end of needle */
		if(*nptr == 0)
			{
			/* found match */
			return pptr;
			} 

		/* test for end of haystack */
		if(*hptr == 0) break;

		/* test for any differences */
		if( toupper(*hptr) != toupper(*nptr) ) break;		

		/* increment positions in strings */
		hptr++;
		nptr++;

		}/* end while all characters in needle */

	/* try next position in haystack */
	pptr++;

	/* test for end of haystack */
	if(*pptr == 0) break;

	}/* end while all positions in haystack */

return 0;
}/* end function strcasestr */


main(int argc, char **argv)
{
int a, b, c, i, f;
char temp[1024];
struct passwd *userinfo;
int faudio;
FILE *fptr;
FILE *fvideo;
FILE *pptr;
int file_format;
char infilename[4096];
char outfilename[4096];
char *ptr;
int high_quality_flag;
int size;
int outfile_flag;
int keep_aspect_flag;
unsigned long lstart_usecs, lcurrent_usecs, ldiff_usecs;
unsigned long show_time;
int resize_flag;
int loop_flag;
int display_flag;

/*select con io 1 byte at the time*/
setbuf(stdout,NULL);
setbuf(stdin,NULL);

fprintf(stdout, "\nPanteltje ppv-%s\n\n", VERSION);

#ifdef PCTV1000
iopl(3);
tv1000_ini();
#endif

/* set defaults */
debug_flag = 0;
outfile_flag = 0;
show_time = 0;
size = -1; /* auto resize always */
high_quality_flag = 1;
keep_aspect_flag = 1;
resize_flag = 1;
loop_flag = 1;
display_flag = 1;
/* end defaults */

while(1)
	{
	a = getopt(argc, argv, "adhi:lo:pqrs:t:");
	if(a == -1) break;
	switch(a)
		{
		case 'a':
			keep_aspect_flag = 0;
			break;
        case 'd': /* not to conflict with -d display option for x */
			debug_flag = 1;
			break;
		case 'h':
			fprintf(stdout,\
"Usage:\n\
ppv [-a] [-d] [-h] -i filename [-l] [-o outfile.mpg] [-p] [-q] [-r] [-s n] [-t n]
\t-a  do not keep aspect.\n\
\t-d  send debug output to console.\n\
\t-h  help (this help).\n\
\t-i  filename.\n\
\t-l  do not loop.\n\
\t-o  filename.mpg output.\n\
\t-p  no picture, do not access video device.\n\
\t-q  selects low quality (352x288) MPEG-2 coding mode (default is 704x576).\n\
\t-r  do not resize.\n\
\t-s  size (0 = 352x288, 1 = 704x576)\n\
\t-t  milli seconds time picture should remain on screen.\n\
");
			exit(0);	
			break;
		case 'i':
			strcpy(infilename, optarg);
			break;
		case 'l':
			loop_flag = 0;
			break;
		case 'o':
			strcpy(outfilename, optarg);
			outfile_flag = 1;
			break;
		case 'p':
			display_flag = 0;
			break;
		case 'q':
			high_quality_flag = 0;
			break;
		case 'r':
			resize_flag = 0;
			break;
		case 's':
			size = atoi(optarg);
			break;
		case 't':
			show_time = 1000 * atoi(optarg);
			break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stdout, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stdout, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			exit(1);
			break;			
		default:
			break;
		}/* end switch a */
	}/* end while getopt() */

userinfo = getpwuid(getuid() );
if(! userinfo)
	{
	fprintf(stdout, "main(): could not get userinfo\n");
	exit(1);
	}

/* get home directory */ 
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stdout, "main(): could not allocate space for home_dir\n");
	exit(1);
	}

start_timeval = malloc(sizeof(struct timeval) );
if(! start_timeval)
	{
	fprintf(stdout, "main(): start_timeval malloc failed\n");
	exit(1);
	}

current_timeval = malloc(sizeof(struct timeval) );
if(! current_timeval)
	{
	fprintf(stdout, "main(): current_timeval malloc failed\n");
	exit(1);
	}

#define PPM		0
#define MPEG	1

/* get the picture format from the extention */
file_format = PPM;
ptr = strrchr(infilename, '.');
if(ptr)
	{
	if(strcasestr(infilename, "mpg") == ptr + 1) file_format = MPEG;
	}

if(debug_flag)
	{
	fprintf(stdout, "main(): infilename=%s file_format=%d ptr=%lu\n",\
	infilename, file_format, ptr);
	}

if(file_format == MPEG) goto show_it;

/* reformat any format to .ppm */
//sprintf(temp, "convert %s /tmp/ppv-ppm.ppm", infilename);
//if(! execute(temp) ) exit(1);		

if( (! resize_flag) && (! keep_aspect_flag) )
	{
	fprintf(stdout,\
	"Warning, resize and aspect correction disabled,\n\
	picture may be in wrong format,\n\
	omit -r and / or -a flags if problems\n");
	}
	
if(resize_flag)
	{
	/* force sizes if not done later in aspect correction */
	if(! keep_aspect_flag) a = '!';
	else a = ' ';

	if(size == -1) /* auto */
		{
		if(high_quality_flag)
			{
			/* resize the pic to 704x576, this overwrites the original */
			sprintf(temp,\
			"convert -geometry 704x576%c %s /tmp/ppv-ppm.ppm" , a , infilename);
			}
		else
			{
			/* resize the pic to 352x288, this overwrites the original */
			sprintf(temp,\
			"convert -geometry 352x288%c %s /tmp/ppv-ppm.ppm" , a , infilename);
			}
		if(! execute(temp) ) exit(1);		
		} /* end if size == -1 */
	else if(size == 0)
		{
		high_quality_flag = 0;
		}
	else if(size == 1)
		{
		high_quality_flag = 1;
		}
	else
		{
		fprintf(stdout, "unknown option argument -s %d\n", size);

		exit(1);
		}
	} /* end if resize_flag */

if(keep_aspect_flag)
	{
	/* prevent question 'overwrite?' */
	if( unlink("/tmp/ppv-ppm.2.ppm") < 0) {} /* first time no file */

	if(high_quality_flag)
		{
		/* combine with standard 704x576 background image */
		sprintf(temp,\
		"combine -geometry 704+0x576+0! -gravity Center %s/.ppv/black.704x576.jpg /tmp/ppv-ppm.ppm /tmp/ppv-ppm.2.ppm",\
		home_dir);
		}
	else
		{
		/* combine with standard 352x288 background image */
		sprintf(temp,\
		"combine -geometry 352+0x288+0! -gravity Center %s/.ppv/black.352x288.jpg /tmp/ppv-ppm.ppm /tmp/ppv-ppm.2.ppm",\
		home_dir);
		}
	if(! execute(temp) ) exit(1);

	/* move new pic to work file */
	sprintf(temp,\
	"cp -f /tmp/ppv-ppm.2.ppm /tmp/ppv-ppm.ppm");
	if(! execute(temp) ) exit(1);
	} /* end if keep_aspect_flag */

/* 
depending on the picture size specified in the command line, 
use the right .par file to do a mpeg2encode
*/

if(high_quality_flag)
	{
	/* uses as input /tmp/ppv-ppm.ppm as specified in 704-576.par */
	if(debug_flag)
		{
   		 sprintf(temp,\
		"mpeg2encode %s/.ppv/704x576.par /tmp/ppv-mpg.mpg",\
		home_dir);
		}
	else
		{
   		 sprintf(temp,\
		"mpeg2encode %s/.ppv/704x576.par /tmp/ppv-mpg.mpg >&/dev/null",\
		home_dir);
		}
	if(! execute(temp) ) exit(1);		
	} /* end if high_quality_flag */
else /* low quality */
	{
    /* uses as input tmp/ppv-ppm.ppm as specified in 352x288.par */
	if(debug_flag)
		{
	    sprintf(temp,\
		"mpeg2encode %s/.ppv/352x288.par /tmp/ppv-mpg.mpg",
		home_dir);
		}
	else
		{
	    sprintf(temp,\
		"mpeg2encode %s/.ppv/352x288.par /tmp/ppv-mpg.mpg >&/dev/null",\
		home_dir);
		}
	if(! execute(temp) ) exit(1);		
	} /* end if ! high_quality_flag */

if(outfile_flag)
	{
	/* append .mpg to output filename if not present */
	ptr = strrchr(outfilename, '.');
	if(ptr && (strcasestr(outfilename, "mpg") ==  ptr + 1) ) {}
	else strcat(outfilename, ".mpg");

	if(debug_flag)
		{
		fprintf(stdout, "main(): outfilename=%s\n", outfilename);
		}
	
	/* copy work file to output fiklename */
	sprintf(temp, "cp /tmp/ppv-mpg.mpg %s", outfilename);
	if(debug_flag)
		{
		fprintf(stdout, "main(): executing=%s\n", temp);
		}
	if(! execute(temp) ) exit(1);		
	} /* end if outfile_flag */

show_it:
if(! display_flag)
	{
#ifdef PCTV1000
	tv1000_set_screen(0);
#endif

	fprintf(stdout, "Ready no display mode.\n");

	/* exit OK */
	exit(0);
	} /* end if ! display_flag */

fvideo = fopen(VIDEODEV, "w");
if(! fvideo)
	{
	fprintf(stdout, "main(): could not open %s for write\n", VIDEODEV);

	exit(1);
	}

if(file_format == MPEG)
	{
	sprintf(temp, "%s", infilename);
	}
else
	{
	sprintf(temp, "/tmp/ppv-mpg.mpg");
	}
	
#ifdef PCTV1000
tv1000_set_screen(1);
#else
fprintf(stdout, "ENTER to exit ppv\n");
#endif

/* set start time */
gettimeofday(start_timeval, NULL);
while(1)
	{
	if( console_status() ) 
		{
		fprintf(stdout, "User abort\n");

		break;
		}

	if(show_time != 0)
		{
		/* get elapsed time */
		gettimeofday(current_timeval, NULL);

		/* calculate time from moment display was last updated */
		lcurrent_usecs =\
			current_timeval -> tv_usec +\
			(1000000 * current_timeval -> tv_sec);

		lstart_usecs =\
			start_timeval -> tv_usec +\
			(1000000 * start_timeval -> tv_sec);

		ldiff_usecs = lcurrent_usecs - lstart_usecs;

		if(ldiff_usecs > show_time)
			{
			fprintf(stdout, "timeout\n");

			break;
			}
		} /* end if show_time != 0 */

	fptr = fopen(temp, "r");
	if(! fptr)
		{
		fprintf(stdout, "main(): could not open file %s for read\n", temp);

#ifdef PCTV1000
		tv1000_set_screen(0);
#endif

		fclose(fvideo);
		exit(1);
		}

	while(1)
		{
		while(1)/* error re read */
			{
			c = getc(fptr);
			if(! ferror(fptr) ) break;
			perror("main(): read failed ");
			}/* end while error re read */	

		if(feof(fptr) || (c == EOF) )
			{
			break;
			}

		fputc(c, fvideo);
		} /* end while read bytes in file */

	fclose(fptr);

	if(! loop_flag)
		{
		fprintf(stdout, "End of file detected, looping disabled\n");

		break;
		} /* end inf ! loop_fag */
	} /* end while read file */


#ifdef PCTV1000
tv1000_set_screen(0);
#endif

fclose(fvideo);

fprintf(stdout, "Ready\n");

/* exit OK */
exit(0);
} /* end function main */


int console_status()
{
int a, c;
fd_set read_file_descr;
struct timeval timeout;

//usleep(100000);
/* this macro initializes the file descriptor read_file_descr to to be the empty
/* set */
FD_ZERO(&read_file_descr);

/* this macro adds fileno(stdin) to the file descriptor read_file_descr */
FD_SET(fileno(stdin), &read_file_descr);

timeout.tv_sec = 0;
timeout.tv_usec = 0;
/* int FD_SETSIZE  macro is maximum number of filedescriptors that
/* fd_set can hold */
/* function select waits for specified filedescr. to have a signal */
/* last argument struct timeval *timeout */
a = select(FD_SETSIZE, &read_file_descr, NULL, NULL, &timeout);
if(a == -1)/* 0 is timeout, -1 error (in errno), 1 = data */
	{
	if(debug_flag) fprintf(stdout, "select returned -1 error\n");
	return 0;
	}
if(a == 0)
	{
//	if(debug_flag) fprintf(stdout, "select returned 0 timeout\n");
	return 0;
	}
if(a == 1)
	{
	if(debug_flag) fprintf(stdout, "SELECT returned=%d input\n", a);
	return 1;
	}

return 1;
}/* end function console_status */



int execute(char *command)
{
FILE *pptr;

if(debug_flag)
	{
	fprintf(stdout, "executing %s\n", command);
	}
pptr = popen(command, "r");
if(pptr <= 0)
	{
	perror("command");

#ifdef PCTV1000
	tv1000_set_screen(0);
#endif

	return 0;
	}

pclose(pptr);
return 1;
}

